﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.Data.Entity;
using System.Threading.Tasks;
using System.Web.Mvc;
using AutoMapper;
using GamesStore.Web.Models.Studios;
using GamesStore.Web.Models.Studios.Index;
using GamesStore.Web.Models.Games;

namespace GamesStore.Web.Controllers
{
    public class StudiosController : BaseGamesController
    {
        // GET: Studios
        public async Task<ViewResult> Index(bool include = false)
        {
            var model = new IndexViewModel()
            {
                OperationStarted = DateTime.Now,
            };

            GamesDataProvider.Database.Log = (query) => model.LogText(query);

            if (include)
            {
                var studios = await GamesDataProvider
                    .Studios
                    .Include(studio => studio.Developers)
                    .Include(studio => studio.Games)
                    .ToListAsync();

                model.Studios = Mapper.Map<IEnumerable<StudioViewModel>>(studios);
            }
            else
            {
                var studios = await GamesDataProvider
                    .Studios
                    .ToListAsync();

                model.Studios = studios
                    .Select(studio => new StudioViewModel(
                        () => studio.Games.Select(game => new GameViewModel()
                        {
                            Id = game.Id,
                            Name = game.Name,
                            Genre = game.Genre,
                            PublishedOn = game.PublishedOn
                        }),
                        () => studio.Developers.Select(developer => new DeveloperViewModel()
                        {
                            Id = developer.Id,
                            FullName = developer.Name
                        }))
                    {
                        Id = studio.Id,
                        Name = studio.Name,
                        FoundationYear = studio.FoundationYear.ToString("0000")
                    });
            }

            return View(model);
        }
    }
}
